/* This file is part of the KDE libraries

   SPDX-FileCopyrightText: 2007 Daniel Laidig <d.laidig@gmx.de>
   SPDX-FileCopyrightText: 2016 John Zaitseff <J.Zaitseff@zap.org.au>

   SPDX-License-Identifier: LGPL-2.0-or-later

   This file is autogenerated by kcharselect/kcharselect-generate-datafile.py
*/

QT_TRANSLATE_NOOP3("KCharSelectData", "All", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "European Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "African Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Middle Eastern Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Central Asian Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "South Asian Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Southeast Asian Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Indonesia and Oceania Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "East Asian Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "American Scripts", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Symbols", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Mathematical Symbols", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Phonetic Symbols", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Diacritics", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Other", "KCharSelect section name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Basic Latin", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin-1 Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "IPA Extensions", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Spacing Modifier Letters", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Diacritical Marks", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Greek and Coptic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cyrillic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cyrillic Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Armenian", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hebrew", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Syriac", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Thaana", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "NKo", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Samaritan", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Mandaic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Syriac Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic Extended-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Devanagari", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Bengali", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Gurmukhi", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Gujarati", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Oriya", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tamil", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Telugu", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Kannada", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Malayalam", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Sinhala", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Thai", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Lao", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tibetan", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Myanmar", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Georgian", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hangul Jamo", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ethiopic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ethiopic Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cherokee", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Unified Canadian Aboriginal Syllabics", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ogham", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Runic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tagalog", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hanunoo", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Buhid", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tagbanwa", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Khmer", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Mongolian", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Unified Canadian Aboriginal Syllabics Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Limbu", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tai Le", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "New Tai Lue", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Khmer Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Buginese", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tai Tham", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Diacritical Marks Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Balinese", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Sundanese", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Batak", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Lepcha", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ol Chiki", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cyrillic Extended-C", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Georgian Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Sundanese Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Vedic Extensions", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Phonetic Extensions", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Phonetic Extensions Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Diacritical Marks Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended Additional", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Greek Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "General Punctuation", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Superscripts and Subscripts", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Currency Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Diacritical Marks for Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Letterlike Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Number Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arrows", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Mathematical Operators", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Technical", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Control Pictures", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Optical Character Recognition", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Enclosed Alphanumerics", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Box Drawing", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Block Elements", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Geometric Shapes", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Dingbats", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Mathematical Symbols-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Arrows-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Braille Patterns", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Arrows-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Mathematical Symbols-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Mathematical Operators", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Symbols and Arrows", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Glagolitic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended-C", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Coptic", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Georgian Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tifinagh", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ethiopic Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cyrillic Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Punctuation", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Radicals Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Kangxi Radicals", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ideographic Description Characters", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Symbols and Punctuation", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hiragana", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Katakana", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Bopomofo", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hangul Compatibility Jamo", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Kanbun", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Bopomofo Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Strokes", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Katakana Phonetic Extensions", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Enclosed CJK Letters and Months", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Compatibility", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Unified Ideographs Extension A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Yijing Hexagram Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Unified Ideographs", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Yi Syllables", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Yi Radicals", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Lisu", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Vai", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cyrillic Extended-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Bamum", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Modifier Tone Letters", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended-D", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Syloti Nagri", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Common Indic Number Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Phags-pa", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Saurashtra", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Devanagari Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Kayah Li", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Rejang", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hangul Jamo Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Javanese", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Myanmar Extended-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cham", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Myanmar Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Tai Viet", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Meetei Mayek Extensions", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ethiopic Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Latin Extended-E", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Cherokee Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Meetei Mayek", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hangul Syllables", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Hangul Jamo Extended-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "High Surrogates", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "High Private Use Surrogates", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Low Surrogates", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Private Use Area", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Compatibility Ideographs", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Alphabetic Presentation Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic Presentation Forms-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Variation Selectors", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Vertical Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Combining Half Marks", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "CJK Compatibility Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Small Form Variants", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Arabic Presentation Forms-B", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Halfwidth and Fullwidth Forms", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Specials", "KCharselect unicode block name");
// initial support for SMP blocks
QT_TRANSLATE_NOOP3("KCharSelectData", "Mahjong Tiles", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Domino Tiles", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Playing Cards", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Enclosed Alphanumeric Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Enclosed Ideographic Supplement", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Miscellaneous Symbols and Pictographs", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Emoticons", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Ornamental Dingbats", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Transport and Map Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Alchemical Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Geometric Shapes Extended", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Arrows-C", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Supplemental Symbols and Pictographs", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Chess Symbols", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Symbols and Pictographs Extended-A", "KCharselect unicode block name");
QT_TRANSLATE_NOOP3("KCharSelectData", "Symbols for Legacy Computing", "KCharselect unicode block name");
